@@ -7,6 +7,9 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $use_PDL);
 BEGIN {
   eval "use PDL::Types; use PDL::Core"; 
   $use_PDL = $@ ? 0 : 1;
+  use vars qw( @ops );
+  @ops = qw( @frame_ops @display_ops @tile_ops @extra_ops
+	   @filetype_ops @scale_ops @mode_ops @orient_ops );
 }
 
 
@@ -16,34 +19,50 @@ require Exporter;
 
 @EXPORT = qw( );
 
-my @frame_ops = qw( 
+use vars @ops, map { "${_}_dbg" } @ops;
+
+@frame_ops = qw( 
+		   FR_active
+		   FR_all
 		   FR_center
 		   FR_clear
 		   FR_delete
-		   FR_hide
-		   FR_new
-		   FR_refresh
-		   FR_reset
 		   FR_first
+		   FR_hide
+		   FR_last
+		   FR_new
 		   FR_next
 		   FR_prev
-		   FR_last
+		   FR_refresh
+		   FR_reset
+		   FR_show
 		  );
 
-my @display_ops = qw( D_blink D_tile D_single );
+@display_ops = qw( D_blink D_tile D_single );
 
-my @tile_ops  = qw( T_Grid T_Column T_Row );
+@tile_ops  = qw( T_Grid T_Column T_Row );
 
-my @extra_ops = qw( ON OFF YES NO );
+@extra_ops = qw( ON OFF YES NO );
 
-my @filetype_ops = qw( FT_MosaicImage FT_MosaicImages FT_Mosaic FT_Array );
+@filetype_ops = qw( FT_MosaicImage FT_MosaicImages FT_Mosaic FT_Array );
 
-my @scale_ops = qw( S_linear S_ln S_log S_squared 
-		    S_sqrt S_minmax S_zscale S_user S_local S_global );
+@scale_ops = qw( S_linear S_log S_squared S_sqrt S_minmax S_zscale
+		 S_user S_local S_global S_limits S_mode S_scope );
 
+@mode_ops = qw( MB_pointer MB_crosshair MB_colorbar MB_pan
+		MB_zoom MB_rotate MB_examine );
+
+@orient_ops = qw( OR_X OR_Y OR_XY );
+
+use vars qw( @all_ops );
+
+eval "push \@all_ops, $_" foreach @ops;
+
+for my $op ( @ops )
+{
+  eval "push ${op}_dbg, eval \$_ foreach ( ${op} )";
+}
 
-@EXPORT_OK = ( @frame_ops, @tile_ops, @extra_ops, @filetype_ops, 
-	       @display_ops, @scale_ops );
 
 %EXPORT_TAGS = ( 
 		frame => \@frame_ops,
@@ -51,9 +70,14 @@ my @scale_ops = qw( S_linear S_ln S_log S_squared
 		filetype => \@filetype_ops,
 		display => \@display_ops,
 		scale => \@scale_ops,
-		all => [ @EXPORT_OK ],
+		mode => \@mode_ops,
+		orient => \@orient_ops,
+		all => \@all_ops,
 	       );
-$VERSION = '0.07';
+
+Exporter::export_ok_tags($_) foreach keys %EXPORT_TAGS;
+
+$VERSION = '0.09';
 
 use Carp;
 use Data::Dumper;
@@ -61,7 +85,6 @@ use IPC::XPA;
 use constant SERVER => 'ds9';
 use constant CLASS => 'Image::DS9';
 
-
 use constant ON		=> 1;
 use constant OFF	=> 0;
 use constant YES	=> 'yes';
@@ -82,7 +105,11 @@ sub _flatten_hash
 # create new XPA object
 {
 
-  my %def_obj_attrs = ( Server => SERVER, min_servers => 1 );
+  my %def_obj_attrs = ( Server => SERVER, 
+			min_servers => 1,
+			res_wanthash => 1,
+			verbose => 0
+		      );
   my %def_xpa_attrs = ( max_servers => 1 );
 
   sub new
@@ -102,7 +129,6 @@ sub _flatten_hash
     
     croak( CLASS, "->new -- error creating XPA object" )
       unless defined $self->{xpa};
-
     
     $self->{xpa_attrs}{max_servers} = $self->nservers || 1;
 
@@ -135,9 +161,7 @@ sub nservers
 
 sub res
 {
-  my $self = shift;
-
-  $self->{res};
+  %{$_[0]->{res}};
 }
 
 {
@@ -165,9 +189,9 @@ sub res
     my ( $self, $image, $attrs ) = @_;
     
     my %attrs = ( $attrs ? %$attrs : () );
-
+    
     my $data = $image;
-
+    
     if ( $use_PDL && 'PDL' eq ref( $image ) )
     {
       $attrs{bitpix} = $map{$image->get_datatype};
@@ -198,10 +222,100 @@ sub display
 {
   my ( $self, $state ) = @_;
 
-  croak( CLASS, '->display -- unknown display type' )
-    if $state ne D_blink && $state ne D_tile && $state ne D_single;
+  unless ( defined $state )
+  {
+    my %blink;
+    my %single;
+    my %tile;
 
-  $self->Set( $state );
+    my $attrs = { chomp => 1, res_wanthash => 1 };
+
+    # catch all of the exceptions and work around them
+    # so as to get maximum data back to caller
+    eval { %blink  = $self->Get( 'blink', $attrs )  };
+    %blink = $self->res if $@;
+
+    eval { %single = $self->Get( 'single', $attrs ) };
+    %single = $self->res if $@;
+
+    eval { %tile   = $self->Get( 'tile', $attrs )   };
+    %tile = $self->res if $@;
+    
+    # create union of server list
+    my %servers;
+    $servers{$_}++ foreach ( keys(%blink), 
+			     keys(%single), 
+			     keys(%tile) );
+    my @servers = keys %servers;
+
+    my %results;
+
+    foreach my $server ( @servers )
+    {
+      my @messages;
+
+      unless ( exists $single{$server} &&
+	       exists $tile{$server} &&
+	       exists $blink{$server} ) 
+      {
+	$results{$server}= { 
+			     name => $server,
+			     message => 
+			     "server did not exist during part of operation" };
+	next;
+      }
+      
+      for my $what ( \%blink, \%single, \%tile )
+      {
+	push @messages, $what->{$server}{message}
+	  if exists $what->{$server}{message};
+      }
+
+      if ( @messages )
+      {
+	$results{$server} = {
+			      name => $server,
+			      message => join( '; ', @messages ) };
+	next;
+      }
+
+      $results{$server} = { 
+			   name => $server,
+			   buf => 
+			    $blink{$server}{buf}  eq 'yes' ? D_blink : 
+			    $single{$server}{buf} eq 'yes' ? D_single :
+			    $tile{$server}{buf}   eq 'yes' ? D_tile :
+			      'unknown'
+			    };
+    }
+    
+    # handle errors now
+    if ( grep { exists $_->{message} } values %results )
+    {
+      $self->{res} = \%results;
+      croak( CLASS, '->display -- error obtaining status' );
+    }
+
+    unless ( wantarray() )
+    {
+      my ( $server ) = keys %results;
+      return $results{$server}{buf};
+    }
+    
+    else
+    {
+      return %results;
+    }
+  }
+
+  else
+  {
+    
+    croak( CLASS, '->display -- unknown display type' )
+      if $state ne D_blink && $state ne D_tile && $state ne D_single;
+    
+    $self->Set( $state );
+  }
 }
 
 use constant T_Grid	 => 'grid';
@@ -214,7 +328,8 @@ sub tile_mode
 
   unless ( defined $state )
   {
-    return $self->Get( 'tile mode' );
+    return $self->Get( 'tile mode', 
+		     { chomp => 1, res_wanthash => wantarray() } );
   }
 
   else
@@ -230,7 +345,8 @@ sub colormap
 
   unless ( defined $colormap )
   {
-    return $self->Get( 'colormap' );
+    return $self->Get( 'colormap', 
+		     { chomp => 1, res_wanthash => wantarray() } );
   }
 
   else
@@ -239,17 +355,36 @@ sub colormap
   }
 }
 
+sub iconify
+{
+  my ( $self, $state ) = @_;
+
+  unless ( defined $state )
+  {
+    return $self->Get( 'iconify', 
+		     { chomp => 1, res_wanthash => wantarray() } );
+  }
+
+  else
+  {
+    $self->Set( "iconify " . bool2str(str2bool($state)) );
+  }
+}
+
+use constant FR_active	=> 'active';
+use constant FR_all	=> 'all';
 use constant FR_center  => 'center';
-use constant FR_clear	 => 'clear';
+use constant FR_clear	=> 'clear';
 use constant FR_delete  => 'delete';
+use constant FR_first	=> 'first';
 use constant FR_hide    => 'hide';
+use constant FR_last	=> 'last';
 use constant FR_new     => 'new';
+use constant FR_next	=> 'next';
+use constant FR_prev	=> 'prev';
 use constant FR_refresh => 'refresh';
 use constant FR_reset   => 'reset';
-use constant FR_first	 => 'first';
-use constant FR_next	 => 'next';
-use constant FR_prev	 => 'prev';
-use constant FR_last	 => 'last';
+use constant FR_show	=> 'show';
 
 sub frame
 {
@@ -258,15 +393,24 @@ sub frame
 
   unless( defined $cmd )
   {
-    return $self->Get( 'frame' );
+    return $self->Get( 'frame', 
+		     { chomp => 1, res_wanthash => wantarray() } );
   }
 
   elsif ( 'show' eq $cmd )
   {
     my $frame = shift;
+    croak( CLASS, '->frame -- too few arguments' )
+      unless defined $frame;
     $self->Set( "frame show $frame" );
   }
 
+  elsif ( 'delete' eq $cmd )
+  {
+    my $frame = shift || '';
+    $self->Set( "frame delete $frame" );
+  }
+
   else
   {
     croak( CLASS, '->frame -- too many arguments' )
@@ -288,7 +432,8 @@ sub file
 
   unless( defined $file )
   {
-    return $self->Get( 'file' );
+    return $self->Get( 'file',
+		     { chomp => 1, res_wanthash => wantarray() } );
   }
 
   else
@@ -300,8 +445,87 @@ sub file
 
 }
 
+use constant MB_pointer		=> 'pointer';
+use constant MB_crosshair	=> 'crosshair';
+use constant MB_colorbar	=> 'colorbar';
+use constant MB_pan		=> 'pan';
+use constant MB_zoom		=> 'zoom';
+use constant MB_rotate		=> 'rotate';
+use constant MB_examine		=> 'examine';
+
+sub mode
+{
+  my ( $self, $state ) = @_;
+
+  unless ( defined $state )
+  {
+    return $self->Get( 'mode', 
+		     { chomp => 1, res_wanthash => wantarray() } );
+  }
+
+  else
+  {
+    $self->Set( "mode $state" );
+  }
+}
+
+
+use constant OR_X	=> 'x';
+use constant OR_Y	=> 'y';
+use constant OR_XY	=> 'xy';
+
+sub orient
+{
+  my ( $self, $state ) = @_;
+
+  unless ( defined $state )
+  {
+    return $self->Get( 'orient', 
+		     { chomp => 1, res_wanthash => wantarray() } );
+  }
+
+  else
+  {
+    $self->Set( "orient $state" );
+  }
+}
+
+sub rotate
+{
+  my $self = shift;
+  my $what = shift;
+
+  unless ( defined $what )
+  {
+    return $self->Get( 'rotate', 
+		     { chomp => 1, res_wanthash => wantarray() } );
+  }
+
+  elsif ( 'abs' eq $what )
+  {
+    $what = shift;
+    croak( CLASS, "->rotate: not enough arguments\n" )
+      unless defined $what;
+    $self->Set( "rotate to $what" );
+  }
+
+  elsif ( 'rel' eq $what )
+  {
+    $what = shift;
+    croak( CLASS, "->rotate: not enough arguments\n" )
+      unless defined $what;
+    $self->Set( "rotate $what" );
+  }
+
+  else
+  {
+    croak( CLASS, "->rotate: too many arguments\n" )
+      if @_;
+    $self->Set( "rotate $what" );
+  }
+}
+
 use constant S_linear	=> 'linear';
-use constant S_ln	=> 'ln';
 use constant S_log	=> 'log';
 use constant S_squared	=> 'squared';
 use constant S_sqrt	=> 'sqrt';
@@ -310,41 +534,74 @@ use constant S_zscale	=> 'zscale';
 use constant S_user	=> 'user';
 use constant S_local	=> 'local';
 use constant S_global	=> 'global';
+use constant S_limits	=> 'limits';
+use constant S_mode	=> 'mode';
+use constant S_scope	=> 'scope';
 
-my @scale_scopes = ( S_local, S_global );
-my @scale_modes = ( S_minmax, S_zscale, S_user );
-my @scale_algs = ( S_linear, S_ln, S_log, S_squared, S_sqrt );
+use vars qw( @scale_scopes @scale_modes @scale_algs );
+@scale_scopes = ( S_local, S_global );
+@scale_modes = ( S_minmax, S_zscale, S_user );
+@scale_algs = ( S_linear, S_log, S_squared, S_sqrt );
 
 sub scale
 {
   my $self = shift;
   my $what = shift;
 
-  if ( 'scope' eq $what )
+  unless ( defined $what )
   {
-    my $scope = shift;
-    grep { $_ eq $scope } @scale_scopes
-      or croak( "unknown scale scope value" );
-
-    $self->Set( "scale scope $scope" );
+    return $self->Get( 'scale', 
+		     { chomp => 1, res_wanthash => wantarray() } );
   }
+
+  elsif ( 'scope' eq $what )
+  {
+    my $what = shift;
+
+    unless ( defined $what )
+    {
+      return $self->Get( 'scale scope', 
+		       { chomp => 1, res_wanthash => wantarray() } );
+    }
+
+    grep { $_ eq $what } @scale_scopes
+      or croak( "unknown scale scope value: `$what'" );
+
+    $self->Set( "scale scope $what" );
+  }
+
   elsif ( 'limits' eq $what )
   {
-    my $minmax = shift;
-    croak ( 'expected array ref for scale limit value' )
-      unless 'ARRAY' eq ref($minmax);
-    croak ( 'not enough values for scale limits' )
-      unless $#{$minmax} >= 2;
+    my $what = shift;
 
-    $self->Set( "scale limits $minmax->[0] $minmax->[1]" );
+    unless ( defined $what )
+    {
+      return $self->Get( 'scale limits', 
+		       { chomp => 1, res_wanthash => wantarray() } );
+    }
+
+    croak ( 'expected array ref for scale limit value' )
+      unless 'ARRAY' eq ref($what);
+    croak ( 'not enough values for scale limits' )
+      unless $#{$what} >= 1;
+
+    $self->Set( "scale limits $what->[0] $what->[1]" );
   }
+
   elsif( 'mode' eq $what )
   {
-    my $mode = shift;
-    grep { $_ eq $mode } @scale_modes
-      or croak( "unknown scale mode value" );
+    my $what = shift;
 
-    $self->Set( "scale mode $mode" );
+    unless ( defined $what )
+    {
+      return $self->Get( 'scale mode', 
+		       { chomp => 1, res_wanthash => wantarray() } );
+    }
+
+    grep { $_ eq $what } @scale_modes
+      or croak( "unknown scale mode value: `$what'" );
+
+    $self->Set( "scale mode $what" );
   }
   else
   {
@@ -359,20 +616,37 @@ sub zoom
   my $self = shift;
   my $what = shift;
 
-  if ( 'abs' eq $what )
+  unless ( defined $what )
   {
+    return $self->Get( 'zoom', 
+		     { chomp => 1, res_wanthash => wantarray() } );
+  }
+
+  elsif ( 'abs' eq $what )
+  {
+    $what = shift;
+    croak( CLASS, "->zoom: not enough arguments\n" )
+      unless defined $what;
     $self->Set( "zoom to $what" );
   }
+
   elsif ( 'rel' eq $what )
   {
+    $what = shift;
+    croak( CLASS, "->zoom: not enough arguments\n" )
+      unless defined $what;
     $self->Set( "zoom $what" );
   }
+
   elsif ( 0 == $what )
   {
     $self->Set( "zoom to fit" );
   }
+
   else
   {
+    croak( CLASS, "->zoom: too many arguments\n" )
+      if @_;
     $self->Set( "zoom to $what" );
   }
 }
@@ -381,49 +655,99 @@ sub Set
 {
   my ( $self, $cmd, $buf ) = @_;
 
-  $self->{res} = [
-    $self->{xpa}->Set( $self->{Server}, $cmd, $buf, $self->{xpa_attrs} ) ];
-  if ( grep { defined $_->{message} } @{$self->{res}} )
+  print STDERR ( CLASS, "->Set: $cmd\n" )
+    if $self->{verbose};
+
+  my %res = $self->{xpa}->Set( $self->{Server}, $cmd, $buf, 
+					    $self->{xpa_attrs} );
+
+  # chomp messages
+  foreach my $res ( values %res )
   {
+    chomp $res->{message} if exists $res->{message};
+  }
+
+  if ( grep { defined $_->{message} } values %res )
+  {
+    $self->{res} = \%res;
     croak( CLASS, " -- error sending data to server" );
   }
 
-  croak( CLASS, " -- fewer than ",$self->{min_servers}," server(s) responded" )
-    if @{$self->{res}} < $self->{min_servers};
+  if ( keys %res < $self->{min_servers} )
+  {
+    $self->{res} = \%res;
+    croak( CLASS, " -- fewer than ", $self->{min_servers}, 
+	   " server(s) responded" )
+  }
 }
 
 sub Get
 {
-  my ( $self, $cmd ) = @_;
-  my @res = $self->{xpa}->Get( $self->{Server}, $cmd, $self->{xpa_attrs} );
-  if ( grep { defined $_->{message} } @res )
+  my ( $self, $cmd, $attr ) = @_;
+
+  print STDERR ( CLASS, "->Get: $cmd\n" )
+    if $self->{verbose};
+
+  my %attr = ( $attr ? %$attr : () );
+
+  $attr{res_wanthash} = $self->{res_wanthash} 
+    unless defined $attr{res_wanthash};
+
+  my %res = $self->{xpa}->Get( $self->{Server}, $cmd, 
+			       $self->{xpa_attrs} );
+
+  # chomp results
+  foreach my $res ( values %res )
   {
-    $self->{res} = \@res;
+    chomp $res->{message} if exists $res->{message};
+    chomp $res->{buf} if exists $res->{buf} && exists $attr{chomp};
+  }
+
+  if ( grep { defined $_->{message} } values %res )
+  {
+    $self->{res} = \%res;
     croak( CLASS, " -- error sending data to server" );
   }
   
-  croak( CLASS, " -- fewer than ",$self->{min_servers}," servers(s) responded" )
-    if @res < $self->{min_servers};
-
-  if ( 1 == $self->{xpa_attrs}{max_servers} )
+  if ( keys %res < $self->{min_servers} )
   {
-    chomp $res[0]->{buf};
-    return $res[0]->{buf};
+    $self->{res} = \%res;
+    croak( CLASS, " -- fewer than ", $self->{min_servers},
+	   " servers(s) responded" )
   }
+
+  unless ( $attr{res_wanthash} )
+  {
+    my ( $server ) = keys %res;
+    return $res{$server}->{buf};
+  }
+
   else
   {
-    return map { chomp $_->{buf}; 
-	         { name => $_->{name}, buf => $_->{buf} } } @res;
+    return %res;
   }
 }
 
 
+sub str2bool
+{
+  my $string = lc shift;
+
+  $string eq 'yes' or $string eq 'on' or $string eq '1';
+}
+
+sub bool2str
+{
+  my $bool = shift;
+  $bool ? 'yes' : 'no';
+}
 
 # Autoload methods go after =cut, and are processed by the autosplit program.
 
 1;
 __END__
-# Below is the stub of documentation for your module. You better edit it!
+
+=pod
 
 =head1 NAME
 
@@ -432,27 +756,12 @@ Image::DS9 - interface to the DS9 image display and analysis program
 =head1 SYNOPSIS
 
   use Image::DS9;
-  use Image::DS9 qw( :frame :tile :filetype );
-  use Image::DS9 qw( :all );
+  use Image::DS9 qw( :<group> );  # import constants from group <group>
+  use Image::DS9 qw( :all );	  # import all constants
 
   $dsp = new Image::DS9;
   $dsp = new Image::DS9( \%attrs );
 
-  $nservers = $dsp->nservers;
-
-  $dsp->array( $image );
-  $dsp->array( $image, \%attrs );
-
-  $dsp->blink( $state );
-
-  $dsp->colormap( $colormap );
-  @colormaps = $dsp->colormap;
-
-  $dsp->frame( $frame_op );
-  @frames = $dsp->frame;
-
-  $last_results = $dsp->res;
-
 =head1 DESCRIPTION
 
 This class provides access to the B<DS9> image display and analysis
@@ -478,24 +787,41 @@ spelling out how to do this.
 
 =head2 Constants
 
+Commands sent to DS9 are sent as strings.  To prevent typos (and other
+unwanted sideeffects) B<Image::DS9> makes many of the commands and
+subcommands available as Perl constants -- mistype these and the
+compiler will complain, not B<DS9>.
+
 Predefined constants may be imported when the B<Image::DS9> package
 is loaded, by specifying one or more of the following tags:
-C<frame>, C<tile>, C<filetype>, C<all>.  For example:
+C<frame>,
+C<tile>,
+C<filetype>,
+C<display>,
+C<scale>,
+C<mode_ops>,
+C<orient_ops>,
+or C<all>.
+
+For example:
 
 	use Image::DS9 qw( :frame :tile :filetype :display );
 
 The C<frame> group imports
+C<FR_active>,
+C<FR_all>,
 C<FR_center>,
 C<FR_clear>,
 C<FR_delete>,
-C<FR_hide>,
-C<FR_new>,
-C<FR_refresh>,
-C<FR_reset>,
 C<FR_first>,
+C<FR_hide>,
+C<FR_last>,
+C<FR_new>,
 C<FR_next>,
 C<FR_prev>,
-C<FR_last>.
+C<FR_refresh>,
+C<FR_reset>,
+C<FR_show>.
 
 The C<tile> group imports
 C<T_Grid>,
@@ -515,7 +841,6 @@ C<D_single>.
 
 The C<scale> group imports
 C<S_linear>,
-C<S_ln>,
 C<S_log>,
 C<S_squared>,
 C<S_sqrt>,
@@ -523,8 +848,24 @@ C<S_minmax>,
 C<S_zscale>,
 C<S_user>,
 C<S_local>,
-C<S_global>.
+C<S_global>,
+C<S_limits>,
+C<S_mode>,
+C<S_scope>.
 
+The C<mode_ops> group imports
+C<MB_pointer>,
+C<MB_crosshair>,
+C<MB_colorbar>,
+C<MB_pan>,
+C<MB_zoom>,
+C<MB_rotate>,
+C<MB_examine>.
+
+The C<orient-ops> group imports
+C<OR_X>,
+C<OR_Y>,
+C<OR_XY>.
 
 The C<all> group imports all of the above groups, as well as
 C<ON>,
@@ -532,39 +873,61 @@ C<OFF>,
 C<YES>
 C<NO>.
 
+=head2 Boolean values
+
+Some methods take boolean values; these may be the strings C<on>, C<off>,
+C<yes>, C<no>, or the integers C<1> or C<0>.
+
 
 =head2 Return values
 
 Because a single B<Image::DS9> object may communicate with multiple
-instances of B<DS9>, most return values are lists, rather than scalars.
-These are lists of hashes, with keys C<name> and C<buf>.  For example,
+instances of B<DS9>, most return values are hashes, rather
+than scalars.  The hash has as keys the names of the servers, with the
+values being references to hashes with the keys C<name>, C<buf> and C<message>.
+The C<buf> key will be present if there are no errors for that server,
+the C<message> if there were. 
+
+For example,
 
 	use Data::Dumper;
-	@colormaps = $dsp->colormap;
+	%colormaps = $dsp->colormap;
 	print Dumper \@colormaps;
 
 yields
 
-	$VAR1 = [
+	$VAR1 = {
+	         'DS9:ds9 838e2ab4:32832' =>
 	          {
 	            'name' => 'DS9:ds9 838e2ab4:32832',
-	            'buf' => 'Grey
-	'
+	            'buf' => 'Grey'
 	          }
-	        ];
+	        };
 
-Note the end of line character in the colormap name.
-
-B<However>, if the object was created with B<max_servers> set to 1,
-it returns the contents of C<buf> directly, i.e.
+If you know that there is only one server out there  (for example,
+if the object was created with B<max_servers> set to 1), you can
+call a method in a scalar environment, and it will directly return
+the value:
 
 	$colormap = $dsp->colormap;
 
+If there is more than one server, you'll get the results for a randomly
+chosen server.
+
+Sending data usually doesn't result in a return:
+
+	$dsp->colormap( 'Grey' );
+
+
 =head2 Error Returns
 
-In case of error, an exception is thrown, and the results
-from the XPA call which failed are made available via the B<res>
-method.
+In case of error, an exception is thrown via B<croak()>.  
+The B<res()> method will return a hash, keyed off of the servers name.
+For each server which had an error, the hash value will be a reference
+to a hash containing the keys C<name> and C<message>; the latter
+will contain error information.  For those commands which return
+data, and for those servers which did not have an error, the
+C<buf> key will be available.
 
 =head2 Methods
 
@@ -598,7 +961,6 @@ The minimum number of servers which should respond to commands.  If
 a response is not received from at least this many servers, an exception
 will be thrown.  It defaults to C<1>.
 
-
 =back
 
 For example,
@@ -642,19 +1004,43 @@ number.
 
 =back
 
+=item colormap
+
+  $dsp->colormap( $colormap );
+  @colormaps = $dsp->colormap;
+
+If an argument is specified, it should be the name of a colormap (case
+is not important).  If no argument is specified, the current colormaps
+for all of the B<DS9> instances is returned, as a list containing
+references to hashes with the keys C<name> and C<buf>.  The latter
+will contain the colormap name.
+
+
 =item display
 
   $dsp->display( $state );
+  %displays = $dsp->display;
+
+If an argument is specified, this call will change how the data are
+displayed. C<$state> may be one of the constants C<D_blink>,
+C<D_tile>, or C<D_single> (or, equivalently, 'blink', 'tile', 'single'
+).  The constants are available by importing the C<display> tag.
+
+If no argument is specified, the current display states of the B<DS9> servers 
+are returned (see L</Return values> for the format).  The state is
+will be returned as a string equivalent to the constants C<D_blink>,
+C<D_tile> or C<D_single>.  For instance:
+
+  my $ds9 = new DS9( { max_servers => 1 } );
+
+  print "We're blinking!\n" if D_blink eq $ds9->display;
 
-Change how frames are displayed.  C<$state> may be one of the constants
-C<D_blink>, C<D_tile>, or C<D_single> (or, equivalently, 
-'blink', 'tile', 'single' ).  The constants are available
-by importing the C<display> tag.
 
 =item file
 
   $dsp->file( $file );
   $dsp->file( $file, $type );
+  %files = $dsp->file;
 
 Display the specified C<$file>.  The file type is optional, and may be
 one of the following constants: C<FT_MosaicImage>, C<FT_MosaicImages>,
@@ -663,43 +1049,36 @@ C<'mosaicimages'>, C<'mosaic'>, or C<'array'> ). (Import the C<filetype>
 tag to get the constants).
 
 If called without a value, it will return the current file name loaded
-for the curent frame.
-
-=item tile_mode
-
-  $dsp->tile_mode( $mode );
-
-The tiling mode may be specified by setting C<$mode> to C<T_Grid>,
-C<T_Column>, or C<T_Row>.  These constants are available if the
-C<tile_op> tags are imported.  Otherwise, use C<'grid'>, c<'column'>,
-or C<'row'>.  If called without a value, it will return the
-current tiling mode.
-
-
-=item colormap
-
-  $dsp->colormap( $colormap );
-  @colormaps = $dsp->colormap;
-
-If an argument is specified, it should be the name of a colormap (case
-is not important).  If no arguments are specified, the current colormaps
-for all of the B<DS9> instances is returned, as a list containing
-references to hashes with the keys C<name> and C<buf>.  The latter
-will contain the colormap name.
+for the current frame.
 
 
 =item frame
 
-  $dsp->frame( $frame_op );
-  $dsp->frame( show => $frame );
   @frames = $dsp->frame;
 
+  # perform a frame operation with no arguments
+  $dsp->frame( $frame_op );
+
+  # show the specified frame
+  $dsp->frame( show => $frame );
+
+  # delete the current frame
+  $dsp->frame( FR_delete );
+    
+  # delete the specified frame
+  $dsp->frame( delete => $frame );
+
+  # delete all of the frames
+  $dsp->frame( delete => FR_all );
+
 Command B<DS9> to do frame operations.  Frame operations are nominally
 strings.  As B<DS9> will interpret any string which isn't a frame operation
 as the name of frame to switch to (or create, if necessary), B<Image::DS9>
 provides constants for the standard operations to prevent typos.  See
-the L<Constants> section.
+L</Constants>.
 Otherwise, use the strings 
+C<active>,
+C<all>,
 C<center>,
 C<clear>,
 C<delete>,
@@ -707,6 +1086,7 @@ C<hide>,
 C<new>,
 C<refresh>,
 C<reset>,
+C<show>,
 C<first>,
 C<next>,
 C<prev>,
@@ -729,6 +1109,46 @@ For example,
 If B<frame()> is called with no arguments, it returns a list of the
 current frames for all instances of B<DS9>.
 
+=item iconify
+
+  $iconify_state = $dsp->iconify;
+  %iconify_state = $dsp->iconify;
+  $dsp->iconify($bool);
+
+With a boolean argument, specify the iconification state, else
+return it.
+
+=item mode
+
+  $mode = $dsp->mode;
+  $dsp->mode( $state );
+
+Change (or query) the first mouse button mode state.  Predefined
+states are available via the C<mode_ops> group; see L</Constants>.
+
+=item orient
+
+  $state = $dsp->orient;
+  $dsp->orient( $state );
+
+Change (or query) the orientation of the current frame. Predefined
+states are available via the C<orient_ops> group; see L</Constants>.
+
+=item rotate
+
+  $rotate = $dsp->rotate;
+  $dsp->rotate( abs => $rotate );
+  $dsp->rotate( rel => $rotate );
+  $dsp->rotate( $rotate );
+
+Change or query the rotation angle (in degrees) for the current frame.
+A rotatation may be absolute or relative; this is explicitly specified
+in the second and third forms of the method invocation.  If not
+specified (as in the last form) it is relative.
+
+If no argument is specified, it returns the rotatation angle for the current
+frame.
+
 =item res
 
   $res = $dsp->res;
@@ -743,20 +1163,23 @@ the hashes contain.
 =item scale
 
   $dsp->scale( $algorithm );
-  $dsp->scale( limits => [ $min, $max ] );
-  $dsp->scale( mode => $mode );
-  $dsp->scale( scope => $scope );
+  $dsp->scale( S_limits => [ $min, $max ] );
+  $dsp->scale( S_mode => $mode );
+  $dsp->scale( S_scope => $scope );
+
+  %scale = $dsp->scale;
+  %limits = $dsp->scale( S_limits );
+  %mode = $dsp->scale( S_mode );
+  %scope = $dsp->scale( S_scope );
 
 This specifies how the data will be scaled.  C<$algorithm> may
 be one of the constants 
 C<S_linear>,
-C<S_ln>,
 C<S_log>,
 C<S_squared>,
 C<S_sqrt>
 (or, equivalently, 
 C<'linear'>,
-C<'ln'>,
 C<'log'>,
 C<'squared'>,
 C<'sqrt'>).
@@ -778,10 +1201,23 @@ C<'local'>,
 C<'global'>
 ).
 
-The constants are available if the C<scale> tag is imported.
+The constants are available if the C<scale> tag is imported (see
+L</Constants>).  The second set of invocations shown above illustrates
+how to determine the current values of the scale parameters.
+
+=item tile_mode
+
+  $dsp->tile_mode( $mode );
+
+The tiling mode may be specified by setting C<$mode> to C<T_Grid>,
+C<T_Column>, or C<T_Row>.  These constants are available if the
+C<tile_op> tags are imported.  Otherwise, use C<'grid'>, c<'column'>,
+or C<'row'>.  If called without a value, it will return the
+current tiling mode.
 
 =item zoom
 
+  $zoom = $dsp->zoom;
   $dsp->zoom( abs => $zoom );
   $dsp->zoom( rel => $zoom );
   $dsp->zoom( $zoom );
@@ -789,10 +1225,12 @@ The constants are available if the C<scale> tag is imported.
 This changes the zoom value for the current frame.  C<$zoom> is a
 positive numerical value.  A zoom value may be absolute or relative.
 This is explicitly specified in the second and third forms of the
-method invocation.  If not specified (as in the third form)
+method invocation.  If not specified (as in the last form)
 it is absolute.  To zoom such that the image fits with in the frame,
 specify a zoom value of C<0>.
 
+If no argument is specified, it returns the zoom value for the current
+frame.
 
 =back
 
